Class: GameMachine::GameSystems::LocalEcho
- Inherits:
-
Actor::Base
- Object
- JavaLib::GameActor
- Actor::Base
- GameMachine::GameSystems::LocalEcho
- Defined in:
- lib/game_machine/game_systems/local_echo.rb
Constant Summary
Constants inherited from Actor::Base
Class Method Summary collapse
-
.echo(message) ⇒ Object
Used in tests to see if actor got the message.
Instance Method Summary collapse
Methods inherited from Actor::Base
aspect, aspects, find, find_by_address, find_distributed, find_distributed_local, find_remote, hashring, local_path, model_filter, #onReceive, player_controller, #receive_message, #schedule_message, #sender, set_player_controller
Class Method Details
.echo(message) ⇒ Object
Used in tests to see if actor got the message
6 7 |
# File 'lib/game_machine/game_systems/local_echo.rb', line 6 def self.echo() end |
Instance Method Details
#on_receive(message) ⇒ Object
9 10 11 12 13 |
# File 'lib/game_machine/game_systems/local_echo.rb', line 9 def on_receive() GameMachine.logger.debug("LocalEcho got #{}") self.class.echo() sender.tell(,self) end |